//***************************************************************************************
// MSP430 DRV8305 1-PWM Motor Driver Control
//
// Description; Using hall sensors installed in a brushless DC motor this code will sense
// the input hall states and apply the necessary output state to the DRV8305.  The driver
// then reads the output states from the MSP430 and supplies the desired output signals
// based on the an internal commutation table to the motor.  This code also samples a speed
// potentiometer and updates the duty cycle of the supplied speed signal.
//
// MSP430G2553
//
// Philip Beard
// Texas Instruments, Inc
// September 2015
// Built with Code Composer Studio v5
//***************************************************************************************

#include <msp430g2553.h>

int HALL_STATE=0;
int Direction=0; // 0 = Clockwise (CW), 1 = Counter Clockwise (CCW)
int Speed=0;
int Current_Speed=25;
int data;
int result;

void Hall_Output();
void Motor_Speed();
void Direction_Change();

int main(void) {
    WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer

	DCOCTL = CALDCO_8MHZ;			// Calibrate DCO to 8 MHz as opposed to default 1 MHz
	BCSCTL1 = CALBC1_8MHZ;

	P2DIR |= BIT0 + BIT1 + BIT6 + BIT7;											// INL_A_DRV8305 Output Pin 2.0
	P2DIR &= (~BIT2)+(~BIT3)+(~BIT4)+(~BIT5);
	P2SEL &= (~BIT0)+(~BIT1)+(~BIT2)+(~BIT3)+(~BIT4)+(~BIT5)+(~BIT7);			// INL_A_DRV8305 GPIO Pin 2.0

	P1DIR |= (BIT4 + BIT3);			// INH_B_DRV8305 and INL_B_DRV8305 Output Pin 2.3 and 2.4
	P1SEL &= (~BIT4 + ~BIT3);		// INH_B_DRV8305 and INL_B_DRV8305 GPIO Pin 2.3 and 2.4
	P1SEL |= BIT5 + BIT6 + BIT7;
	P1SEL2 |= BIT5 + BIT6 + BIT7;
	P1DIR &= (~BIT6);

	P2SEL |= BIT6;					// Route TA0.1 to output pin

	TA0CCR0 = 255; 					// PWM Period, maximum value for Timer A1 (8 MHz / 255 = 31.372 kHz)
	TA0CCTL1 = OUTMOD_7; 			// CCR1 reset/set
	TA0CTL = TASSEL_2 + MC_1;		// Chooses SMCLK and Up Mode

    ADC10CTL1 = INCH_0 + CONSEQ_2;
    ADC10AE0  = BIT0;
	ADC10CTL0 = ADC10ON + MSC; 		// Turn ON ADC and continuously sample.
	ADC10CTL0 |= ENC + ADC10SC; 	// Start Conversion Process

    Speed = ADC10MEM/4;

// -- SPI INITIALIZATION --

// Deactivate State Machine
	UCB0CTL1 = UCSWRST;

// Unselect Device
	P2OUT  |=  BIT7;

// Initialize State Machine

	UCB0CTL0 |=  UCMSB | UCMST | UCSYNC;
	UCB0CTL1 |= UCSSEL_2;   		// SMCLK

// Activate State Machine
	UCB0CTL1 &= ~UCSWRST;

// Select Device
	P2OUT  &=  (~BIT7);

// Write to Register 0x7 to activate 1-PWM Mode

    UCB0TXBUF = 0x3B;
    while (UCB0STAT & BUSY);
    result |= (UCB0RXBUF<< 8);
    while (UCB0STAT & BUSY);
    UCB0TXBUF = 0x16;
    while (UCB0STAT & BUSY);
    result |= UCB0RXBUF;

// Unselect Device
	P2OUT  |=  (BIT7);

// Enable Driver
	P2OUT  |=  BIT1;

	P2IE |= BIT5; 								// Enable Interrupt  on Input Pin P2.5
	P2IES |= BIT5;								// EDGE DETECTION, Falling Edge
	P2IFG &= ~BIT5; 							// P2.5 IFG cleared

	P2IES &= ~BIT3;								// EDGE DETECTION   High = Falling Edge, Low = Rising Edge
	P2IFG &= (~BIT2)+(~BIT3)+(~BIT4); 			// Pin 2.2, 2.3, and 2.4 IFG cleared
	P2IE |=  (BIT2)+(BIT3)+(BIT4) ; 			// Enable Interrupt on Input Pin 2.2, 2.3, and 2.4
	__enable_interrupt();

	P2OUT &= (~BIT2)+(~BIT3)+(~BIT4);

	Hall_Output();
	Motor_Speed();

    while(1){
    	//Hall_Output();
    	//Motor_Speed();
	}
}

#pragma vector=PORT2_VECTOR
__interrupt void    Port_2(void){

	if(P2IFG & BIT5){
		P2OUT &= ~BIT0;
		P1OUT &= ~BIT4 + ~BIT3;
		__delay_cycles(10000000);			// Issue stop motor command for 1.25 seconds
		Direction_Change();
	}
	else if( P2IFG & BIT2 || P2IFG & BIT3 || P2IFG & BIT4){
		Hall_Output();
		Motor_Speed();
		P2IFG &= (~BIT2)+(~BIT3)+(~BIT4);	// Clear Interrupt Flag
	}
	else{
	}
}

void Direction_Change(void){
	if(Direction == 0){
		Direction=1;
		Current_Speed=25;
		TA0CCR1=1;
		Hall_Output();
		Motor_Speed();
		P2IFG &= ~BIT5;			// Clear Interrupt Flag
	}
	else{
		Direction=0;
		Current_Speed=25;
		TA0CCR1=1;
		Hall_Output();
		Motor_Speed();
		P2IFG &= ~BIT5;			// Clear Interrupt Flag
	}
}

void Hall_Output(void){

	HALL_STATE = (P2IN & BIT2) + (P2IN & BIT3) + (P2IN & BIT4);
	switch(HALL_STATE)
	{
	case 4 :
// 		** STATE A **
		if(Direction == 0){
			P1OUT &= ~BIT3;
			P2OUT |= BIT0;
			P1OUT |= BIT4;
			P2IES &= ~BIT4;
		}
		else{
			P2OUT &= ~BIT0;
			P1OUT &= ~BIT4;
			P1OUT |= BIT3;
			P2IES &= ~BIT4;
		}
		break;
	case 12 :
// 		** STATE AB **
		if(Direction == 0){
			P2OUT &= ~BIT0;
			P1OUT &= ~BIT3;
			P1OUT |= BIT4;
			P2IES |= BIT3;
		}
		else{
			P2OUT |= BIT0;
			P1OUT &= ~BIT4;
			P1OUT |=  BIT3;
			P2IES |= BIT3;
		}
		break;
	case 8 :
// 		** STATE B **
		if(Direction == 0){
			P2OUT &= ~BIT0;
			P1OUT |= BIT4 + BIT3;
			P2IES &= ~BIT2;
		}
		else{
			P2OUT |= BIT0;
			P1OUT &= (~BIT4) + (~BIT3);
			P2IES &= ~BIT2;
		}
		break;
	case 24 :
// 		** STATE BC **
		if(Direction == 0){
			P2OUT &= ~BIT0;
			P1OUT &= ~BIT4;
			P1OUT |= BIT3;
			P2IES |= BIT4;
		}
		else{

			P1OUT &= ~BIT3;
			P2OUT |= BIT0;
			P1OUT |= BIT4;
			P2IES |= BIT4;
		}
		break;
	case 16 :
// 		** STATE C **
		if(Direction == 0){
			P1OUT &= ~BIT4;
			P2OUT |= BIT0;
			P1OUT |= BIT3;
			P2IES &= ~BIT3;
		}
		else{
			P2OUT &= ~BIT0;
			P1OUT &= ~BIT3;
			P1OUT |= BIT4;
			P2IES &= ~BIT3;
		}
		break;
	case 20 :
// 		** STATE CA **
		if(Direction == 0){
			P1OUT &= (~BIT4) + (~BIT3);
			P2OUT |= BIT0;
			P2IES |= BIT2;
		}
		else{
			P2OUT &= ~BIT0;
			P1OUT |= BIT3 + BIT4;
			P2IES |= BIT2;
		}
		break;
	}
}
void Motor_Speed(void){
	Speed = ADC10MEM/4;
	if(Current_Speed < Speed){
		Current_Speed=Current_Speed + 2;
		TA0CCR1=Current_Speed;
	}
	else{
		TA0CCR1=Speed;
		Current_Speed=Speed;
	}
}
